var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as React from 'react';
import { lstrings } from '../common/locales/strings';
import { ButtonsModal } from '../components/modals/ButtonsModal';
import { Airship } from '../components/services/AirshipInstance';
import { enableTouchId, isTouchDisabled, isTouchEnabled, supportsTouchId } from '../keychain';
import { hasSecurityAlerts } from '../util/hasSecurityAlerts';
import { showOtpReminder } from '../util/otpReminder';
import { checkAndRequestNotifications } from './LoginInitActions';
/**
 * The user has just logged in, so figure out what do to next.
 */
export const completeLogin = (account) => (dispatch, getState, imports) => __awaiter(void 0, void 0, void 0, function* () {
    // Problem logins:
    const { skipOtpReminder = false, skipSecurityAlerts = false } = imports;
    if (!skipSecurityAlerts && hasSecurityAlerts(account)) {
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'securityAlert', params: { account } }
        });
        return;
    }
    // Recovery logins:
    if (account.recoveryLogin) {
        yield Airship.show(bridge => (React.createElement(ButtonsModal, { bridge: bridge, message: lstrings.recovery_successful, buttons: { ok: { label: lstrings.ok } } })));
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'resecurePassword', params: { account } }
        });
        return;
    }
    // Normal logins:
    if (!skipOtpReminder) {
        yield showOtpReminder(account);
    }
    dispatch(submitLogin(account));
});
/**
 * Everything is done, and we can pass the account to the outside world.
 */
export const submitLogin = (account) => (dispatch, getState, imports) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    const { onLogin } = imports;
    account.watch('loggedIn', loggedIn => {
        if (!loggedIn)
            dispatch({ type: 'RESET_APP' });
    });
    const touchDisabled = yield isTouchDisabled(account);
    if (!touchDisabled) {
        yield enableTouchId(account).catch(e => {
            console.log(e); // Fail quietly
        });
    }
    const isTouchSupported = yield supportsTouchId();
    const touchEnabled = yield isTouchEnabled(account);
    const touchIdInformation = {
        isTouchSupported,
        isTouchEnabled: touchEnabled
    };
    if (onLogin != null)
        onLogin(account, touchIdInformation);
    if (imports.customPermissionsFunction != null) {
        imports.customPermissionsFunction();
    }
    else {
        yield dispatch(checkAndRequestNotifications((_a = imports.branding) !== null && _a !== void 0 ? _a : {})).catch(error => console.log(error));
    }
    // Hide all modals and scenes:
    Airship.clear();
    dispatch({ type: 'NAVIGATE', data: { name: 'loading', params: '' } });
});
