var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { asMaybe } from 'cleaners';
import { makeReactNativeDisklet } from 'disklet';
import * as React from 'react';
import { NativeModules, Platform } from 'react-native';
import { checkNotifications, openSettings, requestNotifications, RESULTS } from 'react-native-permissions';
import { sprintf } from 'sprintf-js';
import { lstrings } from '../common/locales/strings';
import { RequestPermissionsModal } from '../components/modals/RequestPermissionsModal';
import { SecurityAlertsModal } from '../components/modals/SecurityAlertsModal';
import { Airship } from '../components/services/AirshipInstance';
import { arrangeUsers, upgradeUser } from '../hooks/useLocalUsers';
import { scene as sceneReducer } from '../reducers/SceneReducer';
import { asNotificationPermissionsInfo } from '../types/ReduxTypes';
import { launchPasswordRecovery } from './LoginAction';
import { loadTouchState } from './TouchActions';
const { AbcCoreJsUi } = NativeModules;
const disklet = makeReactNativeDisklet();
const notificationPermissionsInfoFile = 'notificationsPermisions.json';
/**
 * Fires off all the things we need to do to get the login scene up & running.
 */
export const initializeLogin = () => (dispatch, getState, imports) => __awaiter(void 0, void 0, void 0, function* () {
    const touchPromise = dispatch(loadTouchState());
    dispatch(checkSecurityMessages()).catch(error => console.log(error));
    yield touchPromise;
    const state = getState();
    // Handle routing using given initialRoute:
    dispatch(routeInitialization(state, imports));
    // Show password recovery if recoveryKey is given:
    const { recoveryKey } = imports;
    if ((imports.initialRoute == null || imports.initialRoute === 'login') &&
        recoveryKey) {
        dispatch(launchPasswordRecovery(recoveryKey));
    }
});
export const maybeRouteComplete = (fallbackAction) => (dispatch, getState, imports) => {
    if (imports.initialRoute == null) {
        dispatch(fallbackAction);
        return;
    }
    const state = getState();
    const sceneState = state.scene;
    const initialRouteSceneName = sceneReducer(sceneState, routeInitialization(state, imports)).name;
    if (imports.onComplete != null &&
        (initialRouteSceneName === sceneState.name ||
            (imports.initialRoute === 'login' && sceneState.name === 'passwordLogin'))) {
        imports.onComplete();
        return;
    }
    dispatch(fallbackAction);
};
/**
 * Loading is done, so send the user to the initial route.
 */
function routeInitialization(state, imports) {
    var _a, _b;
    const { context, initialUserInfo = arrangeUsers(context.localUsers)[0] } = imports;
    const { touch } = state;
    // Try to find the user requested by the LoginScene props:
    const startupUser = initialUserInfo != null ? upgradeUser(initialUserInfo, touch) : undefined;
    const defaultInitialRoute = () => {
        var _a;
        const { recoveryKey } = imports;
        if (recoveryKey) {
            return {
                type: 'NAVIGATE',
                data: { name: 'landing', params: {} }
            };
        }
        else if (startupUser == null) {
            return {
                type: 'NAVIGATE',
                data: { name: 'landing', params: {} }
            };
        }
        else if (startupUser.pinLoginEnabled ||
            (startupUser.touchLoginEnabled && touch.type !== false)) {
            return {
                type: 'NAVIGATE',
                data: {
                    name: 'pinLogin',
                    params: { loginId: startupUser.loginId }
                }
            };
        }
        else {
            return {
                type: 'NAVIGATE',
                data: {
                    name: 'passwordLogin',
                    params: { username: (_a = startupUser.username) !== null && _a !== void 0 ? _a : '' }
                }
            };
        }
    };
    switch (imports.initialRoute) {
        case 'login':
            return defaultInitialRoute();
        case 'login-password':
            return {
                type: 'NAVIGATE',
                data: {
                    name: 'passwordLogin',
                    params: { username: (_a = startupUser === null || startupUser === void 0 ? void 0 : startupUser.username) !== null && _a !== void 0 ? _a : '' }
                }
            };
        case 'login-password-light':
            return {
                type: 'NAVIGATE',
                data: {
                    name: 'passwordLogin',
                    params: {
                        username: (_b = startupUser === null || startupUser === void 0 ? void 0 : startupUser.username) !== null && _b !== void 0 ? _b : '',
                        createAccountType: 'light'
                    }
                }
            };
        case 'new-account':
            return {
                type: 'NAVIGATE',
                data: { name: 'newAccountUsername', params: {} }
            };
        case 'new-light-account': // Set pin, modified TOS
            return {
                type: 'NAVIGATE',
                data: { name: 'newAccountPin', params: {} }
            };
        default:
            return defaultInitialRoute();
    }
}
const checkSecurityMessages = () => (dispatch, getState, imports) => __awaiter(void 0, void 0, void 0, function* () {
    const { context } = imports;
    const messages = yield context.fetchLoginMessages();
    const relevantMessages = [];
    for (const message of messages) {
        if (message.username == null)
            continue;
        // Skip users who haven't fully logged in:
        const info = context.localUsers.find(info => info.username === message.username);
        if (info == null || !info.keyLoginEnabled)
            continue;
        const { otpResetPending, pendingVouchers = [] } = message;
        if (otpResetPending || pendingVouchers.length > 0) {
            relevantMessages.push(message);
        }
    }
    if (relevantMessages.length > 0) {
        const username = yield Airship.show(bridge => (React.createElement(SecurityAlertsModal, { bridge: bridge, messages: relevantMessages })));
        if (username != null) {
            dispatch({
                type: 'NAVIGATE',
                data: { name: 'passwordLogin', params: { username } }
            });
        }
    }
});
//
// Logic for showing notification/app refresh request
//
// 0 = false, 1 = true
// nr = notifications and app refresh request message
// n = notifications request message
// r = app refresh request only message
//
// | Notif Enabled |  Notif Block | App Refresh Enabled || Message |
// |---------------|--------------|---------------------||---------|
// |       0       |       0      |           0         >>    nr   |
// |       0       |       0      |           1         >>    n    |
// |       0       |       1      |           0         >>    r    |
// |       0       |       1      |           1         >>         |
// |       1       |       0      |           0         >>    r    |
// |       1       |       0      |           1         >>         |
// |       1       |       1      |           0         >>    r    |
// |       1       |       1      |           1         >>         |
//
const isIos = Platform.OS === 'ios';
const logicMap = [
    [[], []],
    [[], []]
];
logicMap[0][0][0] = lstrings.notifications_and_refresh_permissions_branded_s;
logicMap[0][0][1] = lstrings.notifications_permissions_branded_s;
logicMap[0][1][0] = lstrings.refresh_permission_branded_s;
logicMap[0][1][1] = undefined;
logicMap[1][0][0] = lstrings.refresh_permission_branded_s;
logicMap[1][0][1] = undefined;
logicMap[1][1][0] = lstrings.refresh_permission_branded_s;
logicMap[1][1][1] = undefined;
export const checkAndRequestNotifications = (branding) => (dispatch, getState, imports) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c, _d;
    const { onNotificationPermit, onLogEvent = () => { } } = imports;
    const notificationPermission = yield checkNotifications();
    const notificationStatus = notificationPermission.status;
    const notifEnabled = notificationStatus !== RESULTS.BLOCKED &&
        notificationStatus !== RESULTS.DENIED
        ? 1
        : 0;
    const statusAppRefresh = isIos
        ? yield AbcCoreJsUi.backgroundAppRefreshStatus().catch((error) => console.log(error))
        : undefined;
    const refreshEnabled = statusAppRefresh !== RESULTS.BLOCKED ? 1 : 0;
    const notificationPermissionsInfoJson = yield disklet
        .getText(notificationPermissionsInfoFile)
        .catch(error => console.log(error));
    const notificationPermissionsInfo = asMaybe(asNotificationPermissionsInfo)(notificationPermissionsInfoJson);
    const isNotificationBlocked = notificationPermissionsInfo
        ? notificationPermissionsInfo.isNotificationBlocked
        : false;
    const isNotificationBlockedBit = isNotificationBlocked ? 1 : 0;
    const permissionMessage = logicMap[notifEnabled][isNotificationBlockedBit][refreshEnabled] != null
        ? sprintf((_a = logicMap[notifEnabled][isNotificationBlockedBit][refreshEnabled]) !== null && _a !== void 0 ? _a : '', (_b = branding.appName) !== null && _b !== void 0 ? _b : lstrings.app_name_default)
        : undefined;
    console.log(`checkAndRequestNotifications`);
    console.log(`notificationStatus:${notificationStatus} statusAppRefresh:${statusAppRefresh}`);
    console.log(`notifEnabled:${notifEnabled} notifBlocked:${isNotificationBlockedBit} refreshEnabled:${refreshEnabled}`);
    console.log(`permissionMessage:${permissionMessage}`);
    if (permissionMessage == null) {
        if (onNotificationPermit != null && notificationPermissionsInfo != null) {
            onNotificationPermit(notificationPermissionsInfo);
        }
        return;
    }
    const choiceMap = {
        optInPriceChanges: !((_c = notificationPermissionsInfo === null || notificationPermissionsInfo === void 0 ? void 0 : notificationPermissionsInfo.notificationOptIns.ignorePriceChanges) !== null && _c !== void 0 ? _c : false),
        optInMarketing: !((_d = notificationPermissionsInfo === null || notificationPermissionsInfo === void 0 ? void 0 : notificationPermissionsInfo.notificationOptIns.ignoreMarketing) !== null && _d !== void 0 ? _d : false)
    };
    const choices = yield Airship.show(bridge => (React.createElement(RequestPermissionsModal, { bridge: bridge, message: permissionMessage, choices: choiceMap })));
    console.log(`checkAndRequestNotifications result ${JSON.stringify(choices)}`);
    // Don't do anything if the user didn't make a choice (dismissed modal)
    if (choices == null)
        return;
    const newPermissionsInfo = {
        isNotificationBlocked: !(choices === null || choices === void 0 ? void 0 : choices.enable),
        notificationOptIns: {
            ignoreMarketing: !choices.optInMarketing,
            ignorePriceChanges: !choices.optInPriceChanges
        }
    };
    yield disklet.setText(notificationPermissionsInfoFile, JSON.stringify(newPermissionsInfo));
    if (choices.enable) {
        if (notificationStatus === RESULTS.DENIED) {
            requestNotifications(isIos ? ['alert', 'badge', 'sound'] : []).catch(error => console.log(error));
        }
        if (notificationStatus === RESULTS.BLOCKED ||
            statusAppRefresh === RESULTS.BLOCKED) {
            openSettings().catch(error => console.log(error));
        }
        onLogEvent('Permission_Modal_Notification_Enable');
    }
    else {
        onLogEvent('Permission_Modal_Notification_Dismiss');
    }
    if (onNotificationPermit != null)
        onNotificationPermit(newPermissionsInfo);
});
