import * as React from 'react';
import { View } from 'react-native';
import { useHandler } from '../../hooks/useHandler';
import { showError } from '../services/AirshipInstance';
import { useTheme } from '../services/ThemeContext';
import { MainButton } from '../themed/MainButton';
import { ModalMessage } from '../themed/ModalParts';
import { ModalUi4 } from '../ui4/ModalUi4';
/**
 * A modal with a title, message, and buttons.
 * This is an alternative to the native `Alert` component.
 *
 * Child components appear between the message and the buttons,
 * but this feature is only meant for inserting extra message elements,
 * like images or custom text formatting.
 *
 * Build a custom modal component if you need form fields, check boxes,
 * or other interactive elements.
 */
export function ButtonsModal(props) {
    const { bridge, title, message, children, buttons, disableCancel = false, fullScreen = false, warning } = props;
    const theme = useTheme();
    const handleCancel = useHandler(() => bridge.resolve(undefined));
    const containerStyle = {
        flex: fullScreen ? 1 : 0
    };
    const textStyle = {
        justifyContent: 'flex-start'
    };
    const buttonsStyle = {
        justifyContent: 'flex-end',
        marginTop: theme.rem(0.5)
    };
    // TODO:
    // Since we don't have clear definitions yet for primary/secondary/tertiary
    // button assignments, we can't use ButtonsViewUi4. For now, just style
    // the buttons with a shared width
    const innerButtonStyle = {
        justifyContent: 'space-between',
        alignSelf: 'center',
        alignItems: 'stretch',
        flexDirection: 'column'
    };
    return (React.createElement(ModalUi4, { warning: warning, bridge: bridge, title: title, onCancel: disableCancel ? undefined : handleCancel },
        React.createElement(View, { style: containerStyle },
            React.createElement(View, { style: textStyle },
                message != null ? React.createElement(ModalMessage, null, message) : null,
                children)),
        React.createElement(View, { style: buttonsStyle },
            React.createElement(View, { style: innerButtonStyle }, Object.keys(buttons).map((key, i, arr) => {
                let defaultType;
                if (theme.preferPrimaryButton) {
                    defaultType = i === 0 ? 'primary' : 'secondary';
                }
                else {
                    defaultType = i === 0 && arr.length > 1 ? 'primary' : 'secondary';
                }
                const { type = defaultType, label, onPress } = buttons[key];
                const handlePress = () => {
                    if (onPress == null) {
                        bridge.resolve(key);
                        return;
                    }
                    return onPress().then(result => {
                        if (result)
                            bridge.resolve(key);
                    }, error => showError(error));
                };
                return (React.createElement(MainButton, { key: key, label: label, marginRem: 0.25, type: type, onPress: handlePress, layout: "column" }));
            })))));
}
