/**
 * IMPORTANT: Changes in this file MUST be duplicated in edge-react-gui!
 */
import DateTimePicker from '@react-native-community/datetimepicker';
import * as React from 'react';
import { Appearance, Platform, Text, TouchableOpacity } from 'react-native';
import { AirshipModal } from 'react-native-airship';
import { lstrings } from '../../common/locales/strings';
import { withTheme } from '../services/ThemeContext';
/**
 * Shows the native iOS date picker inside a modal.
 */
export class DateModalIos extends React.Component {
    constructor(props) {
        super(props);
        this.handleChange = (event, date) => {
            // @ts-expect-error
            this.setState({ date });
        };
        this.handleDone = () => {
            this.props.bridge.resolve(this.state.date);
        };
        this.state = {
            darkMode: Appearance.getColorScheme() === 'dark',
            date: props.initialValue
        };
    }
    componentDidMount() {
        this.subscription = Appearance.addChangeListener(({ colorScheme }) => {
            this.setState({ darkMode: colorScheme === 'dark' });
        });
    }
    componentWillUnmount() {
        if (this.subscription != null)
            this.subscription.remove();
    }
    /**
     * Wrap the data picker component in an Airship modal.
     * This modal doesn't use the normal theme colors,
     * since the native component inside uses its own phone-based colors.
     */
    render() {
        const { bridge, theme } = this.props;
        const { darkMode, date } = this.state;
        const textStyle = {
            color: darkMode ? theme.dateModalTextDark : theme.dateModalTextLight,
            fontSize: theme.rem(1),
            textAlign: 'right',
            paddingHorizontal: theme.rem(1),
            paddingVertical: theme.rem(0.5)
        };
        return (React.createElement(AirshipModal, { bridge: bridge, onCancel: this.handleDone, borderRadius: 0, backgroundColor: darkMode
                ? theme.dateModalBackgroundDark
                : theme.dateModalBackgroundLight },
            React.createElement(TouchableOpacity, { onPress: this.handleDone },
                React.createElement(Text, { style: textStyle }, lstrings.done)),
            React.createElement(DateTimePicker, { display: "spinner", mode: "date", onChange: this.handleChange, value: date })));
    }
}
/**
 * Displays the native Android date picker modal,
 * using the Airship system to manage its lifetime.
 */
export function DateModalAndroid(props) {
    const { bridge, initialValue } = props;
    return (React.createElement(DateTimePicker, { mode: "date", onChange: (event, date) => {
            bridge.resolve(date != null ? date : initialValue);
            bridge.remove();
        }, value: initialValue }));
}
export const DateModal = Platform.OS === 'android' ? DateModalAndroid : withTheme(DateModalIos);
