import * as React from 'react';
import { ActivityIndicator, View } from 'react-native';
import { cacheStyles } from 'react-native-patina';
import { sprintf } from 'sprintf-js';
import { lstrings } from '../../common/locales/strings';
import { useHandler } from '../../hooks/useHandler';
import { QrCode } from '../common/QrCode';
import { showError } from '../services/AirshipInstance';
import { useTheme } from '../services/ThemeContext';
import { ModalMessage } from '../themed/ModalParts';
import { ModalUi4 } from '../ui4/ModalUi4';
export function QrCodeModal(props) {
    const { accountOptions, bridge, context } = props;
    const theme = useTheme();
    const styles = getStyles(theme);
    const [qrData, setQrData] = React.useState();
    const [username, setUsername] = React.useState();
    React.useEffect(() => {
        const cleanups = [];
        let pendingLogin;
        const handleError = (error) => {
            showError(error);
            bridge.resolve(undefined);
        };
        context
            .requestEdgeLogin(Object.assign(Object.assign({}, accountOptions), { 
            // These are no longer used in recent core versions:
            // @ts-expect-error
            displayImageUrl: 'https://github.com/Airbitz/edge-brand-guide/blob/master/Logo/Mark/Edge-Final-Logo_Mark-Green.png', displayName: 'Edge Wallet' }))
            .then(pending => {
            pendingLogin = pending;
            if (pending.state != null) {
                // New core versions have the callbacks on the request:
                pending.watch('state', state => {
                    var _a;
                    if (state === 'started') {
                        setUsername((_a = pending.username) !== null && _a !== void 0 ? _a : lstrings.missing_username);
                    }
                    if (state === 'done' && pending.account != null) {
                        bridge.resolve(pending.account);
                    }
                    if (state === 'error')
                        handleError(pending.error);
                });
            }
            else {
                // Older core versions have the callbacks on the context:
                cleanups.push(context.on(
                // @ts-expect-error
                'login', account => bridge.resolve(account)));
                cleanups.push(context.on(
                // @ts-expect-error
                'loginStart', ({ username }) => setUsername(username)));
                cleanups.push(context.on(
                // @ts-expect-error
                'loginError', ({ error }) => handleError(error)));
            }
            setQrData('edge://edge/' + pending.id);
        });
        return () => {
            // Close the request, ignoring errors (in case it is already closed):
            Promise.resolve(pendingLogin === null || pendingLogin === void 0 ? void 0 : pendingLogin.cancelRequest()).catch(() => { });
            for (const cleanup of cleanups)
                cleanup();
        };
    }, [accountOptions, bridge, context]);
    const handleCancel = useHandler(() => bridge.resolve(undefined));
    return (React.createElement(ModalUi4, { bridge: bridge, onCancel: handleCancel, title: lstrings.qr_modal_title, scroll: true },
        React.createElement(ModalMessage, null, username != null
            ? sprintf(lstrings.qr_modal_started, username)
            : lstrings.qr_modal_message),
        React.createElement(View, { style: styles.qrContainer }, qrData == null ? (React.createElement(ActivityIndicator, { color: theme.primaryText })) : (React.createElement(QrCode, { key: "qrcode", data: qrData, size: theme.rem(14) })))));
}
const getStyles = cacheStyles((theme) => ({
    qrContainer: {
        alignItems: 'center',
        justifyContent: 'center',
        minHeight: theme.rem(16),
        padding: theme.rem(0.5)
    }
}));
