import * as React from 'react';
import { TouchableOpacity, View } from 'react-native';
import { cacheStyles } from 'react-native-patina';
import IonIcon from 'react-native-vector-icons/Ionicons';
import { useTheme } from '../services/ThemeContext';
import { EdgeText } from '../themed/EdgeText';
import { ListModal } from './ListModal';
export function QuestionListModal(props) {
    const { bridge, items, selected, title } = props;
    const theme = useTheme();
    const styles = getStyles(theme);
    function renderRow(item) {
        const radio = {
            icon: `ios-radio-button-${selected === item ? 'on' : 'off'}`,
            color: theme.iconTappable
        };
        return (React.createElement(TouchableOpacity, { onPress: () => bridge.resolve(item) },
            React.createElement(View, { style: styles.row },
                React.createElement(View, { style: styles.textContainer },
                    React.createElement(EdgeText, { style: styles.text, numberOfLines: 0, disableFontScaling: true }, item)),
                React.createElement(IonIcon, { style: styles.radio, name: radio.icon, color: radio.color, size: theme.rem(1.25) }))));
    }
    return (React.createElement(ListModal, { bridge: bridge, title: title, textInput: false, rowsData: items, rowComponent: (item) => renderRow(item), fullScreen: false }));
}
const getStyles = cacheStyles((theme) => ({
    row: {
        alignItems: 'center',
        flexDirection: 'row',
        justifyContent: 'flex-start',
        margin: theme.rem(0.5)
    },
    radio: {
        alignSelf: 'center',
        marginRight: theme.rem(0.25),
        marginLeft: theme.rem(0.375)
    },
    textContainer: {
        flex: 1
    },
    text: {
        fontSize: theme.rem(1)
    }
}));
