/**
 * IMPORTANT: Changes in this file MUST be duplicated in edge-react-gui!
 */
import * as React from 'react';
import { Platform, View } from 'react-native';
import { lstrings } from '../../common/locales/strings';
import { showError } from '../services/AirshipInstance';
import { FilledTextInput } from '../themed/FilledTextInput';
import { MainButton } from '../themed/MainButton';
import { ModalMessage, ModalTitle } from '../themed/ModalParts';
import { AlertCardUi4 } from '../ui4/AlertUi4';
import { ModalUi4 } from '../ui4/ModalUi4';
export function TextInputModal(props) {
    const { autoCapitalize, autoFocus = true, autoCorrect, bridge, initialValue = '', inputLabel, keyboardType, message, onSubmit, returnKeyType, secureTextEntry, multiline = false, submitLabel = lstrings.submit, title, maxLength, warning, warningMessage } = props;
    const [errorMessage, setErrorMessage] = React.useState();
    const [spinning, setSpinning] = React.useState(false);
    const [text, setText] = React.useState(initialValue);
    const handleChangeText = (text) => {
        setText(text);
        setErrorMessage(undefined);
    };
    const handleSubmit = () => {
        if (onSubmit == null)
            return bridge.resolve(text);
        setSpinning(true);
        onSubmit(text).then(result => {
            setSpinning(false);
            if (typeof result === 'string') {
                setErrorMessage(result);
            }
            else if (result) {
                bridge.resolve(text);
            }
        }, error => {
            setSpinning(false);
            showError(error);
        });
    };
    return (React.createElement(ModalUi4, { warning: warning, bridge: bridge, onCancel: () => bridge.resolve(undefined) },
        title != null ? React.createElement(ModalTitle, null, title) : null,
        typeof message === 'string' ? (React.createElement(ModalMessage, null, message)) : (React.createElement(React.Fragment, null, message)),
        warningMessage != null ? (React.createElement(AlertCardUi4, { type: "warning", title: lstrings.warning, marginRem: 0.5, body: warningMessage })) : null,
        React.createElement(FilledTextInput, { top: 1, horizontal: 0.5, bottom: 1.5, 
            // Text input props:
            autoCapitalize: autoCapitalize, autoFocus: autoFocus, autoCorrect: autoCorrect, keyboardType: keyboardType, placeholder: inputLabel, returnKeyType: returnKeyType, secureTextEntry: secureTextEntry, multiline: multiline, 
            // Our props:
            error: errorMessage, onChangeText: handleChangeText, onSubmitEditing: handleSubmit, value: text, maxLength: maxLength }),
        // Hack around the android:windowSoftInputMode="adjustPan" glitch:
        Platform.OS === 'android' ? React.createElement(View, { style: { flex: 2 } }) : null,
        spinning ? (React.createElement(MainButton, { alignSelf: "center", disabled: true, marginRem: 0.5, type: "secondary", spinner: true })) : (React.createElement(MainButton, { alignSelf: "center", label: submitLabel, marginRem: 0.5, onPress: handleSubmit, type: "secondary" }))));
}
