var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as React from 'react';
import { Keyboard, KeyboardAvoidingView } from 'react-native';
import { cacheStyles } from 'react-native-patina';
import { lstrings } from '../../common/locales/strings';
import { useHandler } from '../../hooks/useHandler';
import { useImports } from '../../hooks/useImports';
import { useDispatch } from '../../types/ReduxTypes';
import { EdgeAnim } from '../common/EdgeAnim';
import { WarningCard } from '../common/WarningCard';
import { ButtonsModal } from '../modals/ButtonsModal';
import { Airship, showError } from '../services/AirshipInstance';
import { useTheme } from '../services/ThemeContext';
import { EdgeText } from '../themed/EdgeText';
import { FilledTextInput } from '../themed/FilledTextInput';
import { MainButton } from '../themed/MainButton';
import { PasswordStatus } from '../themed/PasswordStatus';
import { ThemedScene } from '../themed/ThemedScene';
import { AlertCardUi4 } from '../ui4/AlertUi4';
const ChangePasswordSceneComponent = ({ initPassword, title, onBack, onSkip, onSubmit, mainButtonLabel = lstrings.done }) => {
    var _a;
    const theme = useTheme();
    const styles = getStyles(theme);
    const imports = useImports();
    const [focusFirst, setFocusFirst] = React.useState(true);
    const [focusSecond, setFocusSecond] = React.useState(false);
    const [spinning, setSpinning] = React.useState(false);
    const [isShowError, setIsShowError] = React.useState(false);
    const [passwordEval, setPasswordEval] = React.useState(undefined);
    const isRequirementsMet = (_a = passwordEval === null || passwordEval === void 0 ? void 0 : passwordEval.passed) !== null && _a !== void 0 ? _a : false;
    const [password, setPassword] = React.useState(initPassword !== null && initPassword !== void 0 ? initPassword : '');
    const [confirmPassword, setConfirmPassword] = React.useState('');
    const [confirmPasswordErrorMessage, setConfirmPasswordErrorMessage] = React.useState('');
    const handlePress = useHandler(() => __awaiter(void 0, void 0, void 0, function* () {
        if (!isRequirementsMet)
            return;
        if (password !== confirmPassword) {
            setConfirmPasswordErrorMessage(lstrings.password_mismatch_error);
            setIsShowError(true);
            return;
        }
        setSpinning(true);
        try {
            yield onSubmit(password);
        }
        catch (e) {
            showError(e);
            setSpinning(false);
        }
    }));
    const handleFocusSwitch = () => {
        setFocusFirst(false);
        setFocusSecond(true);
    };
    const handleValidatePassword = (password) => {
        setPassword(password);
        setIsShowError(false);
        setPasswordEval(imports.context.checkPasswordRules(password));
    };
    const handleChangeConfirmPassword = (confirmPassword) => {
        setConfirmPassword(confirmPassword);
        setIsShowError(false);
        if (confirmPassword !== password) {
            setConfirmPasswordErrorMessage(lstrings.password_mismatch_error);
            setIsShowError(true);
        }
    };
    const renderInterior = () => {
        return (React.createElement(React.Fragment, null,
            passwordEval != null ? (React.createElement(PasswordStatus, { marginRem: [0.5, 0.5, 1.25], passwordEval: passwordEval })) : (React.createElement(EdgeAnim, { enter: { type: 'fadeInUp', distance: 50 }, exit: { type: 'fadeOutDown' } },
                React.createElement(EdgeText, { style: styles.description, numberOfLines: 4 }, lstrings.password_desc))),
            React.createElement(EdgeAnim, { enter: { type: 'fadeInUp', distance: 25 } },
                React.createElement(FilledTextInput, { horizontal: 1.25, bottom: 1.25, value: password, secureTextEntry: true, returnKeyType: "next", placeholder: lstrings.password, autoFocus: focusFirst, onChangeText: handleValidatePassword, onSubmitEditing: handleFocusSwitch, clearIcon: true, maxLength: 100 })),
            React.createElement(EdgeAnim, { enter: { type: 'fadeInDown', distance: 25 } },
                React.createElement(FilledTextInput, { horizontal: 1.25, bottom: 1.25, value: confirmPassword, secureTextEntry: true, returnKeyType: "go", placeholder: lstrings.confirm_password, autoFocus: focusSecond, onChangeText: handleChangeConfirmPassword, onSubmitEditing: handlePress, clearIcon: true, maxLength: 100 })),
            isShowError ? (React.createElement(EdgeAnim, { style: styles.actions, enter: { type: 'fadeInUp', distance: 50 }, exit: { type: 'fadeOutUp', distance: 50 } },
                React.createElement(AlertCardUi4, { marginRem: [0.75, 1.25], type: "error", title: confirmPasswordErrorMessage }))) : null,
            React.createElement(EdgeAnim, { style: styles.actions, enter: { type: 'fadeInDown', distance: 50 }, exit: { type: 'fadeOutDown', distance: 50 }, visible: !isShowError }, spinning ? (React.createElement(MainButton, { alignSelf: "center", disabled: true, marginRem: 0.5, type: "primary", spinner: true })) : (React.createElement(MainButton, { alignSelf: "center", label: mainButtonLabel, disabled: !isRequirementsMet || confirmPassword === '', marginRem: 0.5, onPress: handlePress, type: "primary" })))));
    };
    return (React.createElement(ThemedScene, { onBack: onBack, onSkip: onSkip, title: title }, focusSecond ? (React.createElement(KeyboardAvoidingView, { style: styles.container, behavior: "position", keyboardVerticalOffset: -150 }, renderInterior())) : (renderInterior())));
};
const getStyles = cacheStyles((theme) => ({
    container: {
        flex: 1,
        marginHorizontal: theme.rem(0.5),
        overflow: 'hidden'
    },
    description: {
        fontFamily: theme.fontFaceDefault,
        fontSize: theme.rem(0.875),
        marginBottom: theme.rem(2),
        marginTop: theme.rem(1),
        marginHorizontal: theme.rem(0.5)
    },
    actions: {
        flexDirection: 'row',
        justifyContent: 'center',
        marginTop: theme.rem(1)
    }
}));
// The scene for existing users to change their password
export const ChangePasswordScene = (props) => {
    const { route } = props;
    const { account } = route.params;
    const { onComplete = () => { } } = useImports();
    const handleSubmit = useHandler((password) => __awaiter(void 0, void 0, void 0, function* () {
        Keyboard.dismiss();
        yield account.changePassword(password);
        yield Airship.show(bridge => (React.createElement(ButtonsModal, { bridge: bridge, title: lstrings.password_changed, message: lstrings.pwd_change_modal, buttons: { ok: { label: lstrings.ok } } },
            React.createElement(WarningCard, { title: lstrings.warning, header: lstrings.password_change_warning, marginRem: [1, 0.5] }))));
        onComplete();
    }));
    return React.createElement(ChangePasswordSceneComponent, { onSubmit: handleSubmit });
};
// The scene for existing users to recover their password
export const ResecurePasswordScene = (props) => {
    const { route } = props;
    const { account } = route.params;
    const dispatch = useDispatch();
    const handleSkip = useHandler(() => {
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'resecurePin', params: { account } }
        });
    });
    const handleSubmit = useHandler((password) => __awaiter(void 0, void 0, void 0, function* () {
        Keyboard.dismiss();
        yield account.changePassword(password);
        yield Airship.show(bridge => (React.createElement(ButtonsModal, { bridge: bridge, title: lstrings.password_changed, message: lstrings.pwd_change_modal, buttons: { ok: { label: lstrings.ok } } })));
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'resecurePin', params: { account } }
        });
    }));
    return (React.createElement(ChangePasswordSceneComponent, { onSkip: handleSkip, title: lstrings.change_password, onSubmit: handleSubmit }));
};
// The scene for new users to create a password
export const NewAccountPasswordScene = (props) => {
    const { route } = props;
    const { onLogEvent = () => { } } = useImports();
    const dispatch = useDispatch();
    const handleBack = useHandler(() => {
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'newAccountUsername', params: route.params }
        });
    });
    const handleSubmit = useHandler((newPassword) => {
        onLogEvent('Signup_Password_Valid');
        dispatch({
            type: 'NAVIGATE',
            data: {
                name: 'newAccountPin',
                params: Object.assign(Object.assign({}, route.params), { password: newPassword })
            }
        });
    });
    return (React.createElement(ChangePasswordSceneComponent, { onBack: handleBack, onSubmit: handleSubmit, title: lstrings.choose_title_password, mainButtonLabel: lstrings.next_label }));
};
// The scene for light account users to create a password for the upgrade/backup
export const UpgradePasswordScene = (props) => {
    const { route } = props;
    const { onLogEvent = () => { } } = useImports();
    const dispatch = useDispatch();
    const handleBack = useHandler(() => {
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'upgradeUsername', params: route.params }
        });
    });
    const handleSubmit = useHandler((newPassword) => __awaiter(void 0, void 0, void 0, function* () {
        onLogEvent('Backup_Password_Valid');
        dispatch({
            type: 'NAVIGATE',
            data: {
                name: 'upgradeTos',
                params: Object.assign(Object.assign({}, route.params), { password: newPassword })
            }
        });
    }));
    return (React.createElement(ChangePasswordSceneComponent, { onBack: handleBack, onSubmit: handleSubmit, title: lstrings.choose_title_password, mainButtonLabel: lstrings.next_label }));
};
