var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { asMaybeOtpError } from 'edge-core-js';
import * as React from 'react';
import FontAwesome from 'react-native-vector-icons/FontAwesome';
import { sprintf } from 'sprintf-js';
import { completeLogin } from '../../../actions/LoginCompleteActions';
import { lstrings } from '../../../common/locales/strings';
import { useHandler } from '../../../hooks/useHandler';
import { useImports } from '../../../hooks/useImports';
import { useDispatch } from '../../../types/ReduxTypes';
import { toLocalTime } from '../../../util/utils';
import { showResetModal } from '../../modals/OtpResetModal';
import { QrCodeModal } from '../../modals/QrCodeModal';
import { TextInputModal } from '../../modals/TextInputModal';
import { Airship, showError } from '../../services/AirshipInstance';
import { DividerWithText } from '../../themed/DividerWithText';
import { IconHeaderRow } from '../../themed/IconHeaderRow';
import { LinkRow } from '../../themed/LinkRow';
import { ThemedScene } from '../../themed/ThemedScene';
import { MessageText, Warning } from '../../themed/ThemedText';
export function OtpRepairScene(props) {
    var _a;
    const { branding, route } = props;
    const { account, otpError } = route.params;
    const { resetToken } = otpError;
    const { accountOptions, context, onComplete = () => { } } = useImports();
    const dispatch = useDispatch();
    const [otpResetDate, setOtpResetDate] = React.useState(otpError.resetDate);
    //
    // Handlers
    //
    const handleBackupModal = useHandler(() => {
        function handleSubmit(otpKey) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    if (account.repairOtp == null) {
                        throw new Error('Wrong edge-core-js version');
                    }
                    yield account.repairOtp(otpKey);
                    onComplete();
                    return true;
                }
                catch (error) {
                    // Translate known errors:
                    const otpError = asMaybeOtpError(error);
                    if (otpError != null) {
                        dispatch({
                            type: 'NAVIGATE',
                            data: { name: 'otpRepair', params: { account, otpError } }
                        });
                        setOtpResetDate(otpError.resetDate);
                        return lstrings.backup_key_incorrect;
                    }
                    if (error instanceof Error &&
                        error.message === 'Unexpected end of data') {
                        return lstrings.backup_key_incorrect;
                    }
                    showError(error);
                    return false;
                }
            });
        }
        Airship.show(bridge => (React.createElement(TextInputModal, { bridge: bridge, autoCapitalize: "characters", inputLabel: lstrings.backup_key_label, message: lstrings.otp_instructions, returnKeyType: "done", submitLabel: lstrings.submit, title: lstrings.otp_backup_code_modal_title, onSubmit: handleSubmit }))).catch(error => showError(error));
    });
    const handleQrModal = useHandler(() => {
        Airship.show(bridge => (React.createElement(QrCodeModal, { bridge: bridge, accountOptions: accountOptions, context: context })))
            .then((account) => __awaiter(this, void 0, void 0, function* () {
            if (account != null)
                yield dispatch(completeLogin(account));
        }))
            .catch(error => showError(error));
    });
    const handleResetModal = useHandler(() => {
        function handleSubmit() {
            return __awaiter(this, void 0, void 0, function* () {
                if (resetToken == null) {
                    throw new Error('No OTP reset token');
                }
                if (account.username == null) {
                    throw new Error('No username');
                }
                const date = yield context.requestOtpReset(account.username, resetToken);
                setOtpResetDate(date);
            });
        }
        showResetModal(handleSubmit).catch(error => showError(error));
    });
    //
    // Render
    //
    const isIp = otpError.reason === 'ip';
    // Find the automatic login date:
    const date = (_a = otpError.voucherActivates) !== null && _a !== void 0 ? _a : otpResetDate;
    return (React.createElement(ThemedScene, { onBack: onComplete, title: lstrings.otp_header_repair },
        React.createElement(IconHeaderRow, { renderIcon: theme => (React.createElement(Warning, null,
                React.createElement(FontAwesome, { name: "exclamation-triangle", size: theme.rem(2.5) }))) },
            React.createElement(MessageText, null,
                React.createElement(Warning, null, isIp
                    ? sprintf(lstrings.otp_repair_header_ip_branded, branding.appName)
                    : sprintf(lstrings.otp_repair_header_2fa_branded, branding.appName)))),
        React.createElement(DividerWithText, { label: lstrings.to_fix }),
        React.createElement(MessageText, null, lstrings.otp_scene_approve),
        React.createElement(DividerWithText, null),
        React.createElement(LinkRow, { label: lstrings.otp_scene_qr, onPress: handleQrModal }),
        isIp ? null : (React.createElement(React.Fragment, null,
            React.createElement(DividerWithText, null),
            React.createElement(LinkRow, { label: lstrings.otp_backup_code_modal_title, onPress: handleBackupModal }))),
        date == null ? null : (React.createElement(React.Fragment, null,
            React.createElement(DividerWithText, null),
            React.createElement(MessageText, null, sprintf(lstrings.otp_scene_wait, toLocalTime(date))))),
        resetToken == null || date != null ? null : (React.createElement(React.Fragment, null,
            React.createElement(DividerWithText, null),
            React.createElement(LinkRow, { label: lstrings.disable_otp_button_two, onPress: handleResetModal })))));
}
