import * as React from 'react';
import { Image, View } from 'react-native';
import { cacheStyles } from 'react-native-patina';
import MaterialCommunityIcons from 'react-native-vector-icons/MaterialCommunityIcons';
import { sprintf } from 'sprintf-js';
import { LOGO_BIG } from '../../../assets';
import { lstrings } from '../../../common/locales/strings';
import * as Constants from '../../../constants/index';
import { useImports } from '../../../hooks/useImports';
import { useLocalUsers } from '../../../hooks/useLocalUsers';
import { useDispatch } from '../../../types/ReduxTypes';
import { useTheme } from '../../services/ThemeContext';
import { DividerLine } from '../../themed/DividerLine';
import { EdgeText } from '../../themed/EdgeText';
import { MainButton } from '../../themed/MainButton';
import { ThemedScene } from '../../themed/ThemedScene';
export const NewAccountWelcomeScene = (props) => {
    const { branding } = props;
    const dispatch = useDispatch();
    const theme = useTheme();
    const styles = getStyles(theme);
    const { experimentConfig, onLogEvent = () => { } } = useImports();
    const localUsers = useLocalUsers();
    const hasSavedUsers = localUsers.length > 0;
    const appName = branding.appName || lstrings.app_name_default;
    const buttonType = theme.preferPrimaryButton ? 'primary' : 'secondary';
    const logoSrc = branding.primaryLogo || LOGO_BIG;
    const handleDone = () => {
        onLogEvent(`Signup_Welcome_Next`);
        dispatch({
            type: 'NAVIGATE',
            data: {
                name: hasSavedUsers || experimentConfig.createAccountType === 'full'
                    ? 'newAccountUsername'
                    : 'newAccountPin',
                params: {}
            }
        });
    };
    const handleExit = () => {
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'landing', params: {} }
        });
    };
    return (React.createElement(ThemedScene, { onBack: handleExit, title: lstrings.get_started },
        React.createElement(View, { style: styles.content },
            React.createElement(Image, { source: logoSrc, style: styles.logo, resizeMode: "contain" }),
            React.createElement(EdgeText, { style: styles.welcome }, sprintf(lstrings.welcome, appName)),
            React.createElement(View, { style: styles.advantage },
                React.createElement(MaterialCommunityIcons, { style: styles.advantageImage, name: "lock", size: theme.rem(2) }),
                React.createElement(View, { style: styles.advantageTextContainer },
                    React.createElement(EdgeText, { style: styles.advantageTitle }, sprintf(lstrings.welcome_advantage_one_title, appName)),
                    React.createElement(EdgeText, { style: styles.advantageDescription, numberOfLines: 2 }, lstrings.welcome_advantage_one_description_line1),
                    React.createElement(EdgeText, { style: styles.advantageDescription, numberOfLines: 2 }, lstrings.welcome_advantage_one_description_line2))),
            React.createElement(DividerLine, { marginRem: [1.5, 0.5] }),
            React.createElement(View, { style: styles.advantage },
                React.createElement(MaterialCommunityIcons, { style: styles.advantageImage, name: "shield-key", size: theme.rem(2) }),
                React.createElement(View, { style: styles.advantageTextContainer },
                    React.createElement(EdgeText, { style: styles.advantageTitle }, lstrings.welcome_advantage_two_title),
                    React.createElement(EdgeText, { style: styles.advantageDescription, numberOfLines: 5 }, sprintf(lstrings.welcome_advantage_two_description, appName)))),
            React.createElement(View, { style: styles.actions },
                React.createElement(MainButton, { label: lstrings.get_started, type: buttonType, onPress: handleDone })))));
};
const getStyles = cacheStyles((theme) => ({
    content: {
        flex: 1,
        alignItems: 'center'
    },
    logo: {
        height: theme.rem(2.25),
        marginTop: theme.rem(0.75),
        marginBottom: theme.rem(1)
    },
    welcome: {
        fontFamily: theme.fontFaceBold,
        color: theme.secondaryText,
        fontSize: theme.rem(1),
        marginBottom: theme.rem(2.375)
    },
    advantage: {
        flexDirection: 'row',
        alignItems: 'center',
        marginHorizontal: theme.rem(0.5)
    },
    advantageImage: {
        color: theme.iconTappable,
        height: theme.rem(2),
        width: theme.rem(2),
        marginRight: theme.rem(0.5)
    },
    advantageTextContainer: {
        flex: 1
    },
    advantageTitle: {
        fontFamily: theme.fontFaceBold,
        color: Constants.GRAY_4,
        fontSize: theme.rem(1),
        marginBottom: theme.rem(0.5)
    },
    advantageDescription: {
        fontFamily: theme.fontFaceDefault,
        fontSize: theme.rem(0.75)
    },
    actions: {
        marginTop: theme.rem(4.5)
    }
}));
