/**
 * IMPORTANT: Changes in this file MUST be synced with edge-react-gui!
 */
import * as React from 'react';
import { View } from 'react-native';
import { cacheStyles } from 'react-native-patina';
import { fixSides, mapSides, sidesToMargin } from '../../util/sides';
import { useTheme } from '../services/ThemeContext';
const DEFAULT_MARGIN_REM = 0.5;
/**
 * View that automatically adds horizontal dividers between each child, aligned
 * in a column layout. Adds no dividers if only one child is given.
 *
 * wideSpacing is meant for sectioning out a scene where more spacing is needed
 * between sections.
 */
export const SectionView = (props) => {
    const { children, extendRight = false, marginRem, dividerVerticalRem } = props;
    const theme = useTheme();
    const styles = getStyles(theme);
    const margin = marginRem != null
        ? sidesToMargin(mapSides(fixSides(marginRem, 0), theme.rem))
        : extendRight
            ? styles.marginScene
            : styles.marginCard;
    const dividerMargin = dividerVerticalRem != null
        ? sidesToMargin(mapSides(fixSides(dividerVerticalRem, 0), theme.rem))
        : extendRight
            ? styles.dividerMarginScene
            : styles.dividerMarginCard;
    const nonNullChildren = React.Children.map(children, child => {
        if (child != null) {
            return child;
        }
    });
    const numChildren = React.Children.count(nonNullChildren);
    if (children == null || numChildren === 0)
        return null;
    // Add a line divider between each child if there's more than one:
    return (React.createElement(View, { style: [styles.container, margin] }, numChildren === 1
        ? nonNullChildren
        : React.Children.map(nonNullChildren, (child, index) => {
            if (index < numChildren - 1) {
                return (React.createElement(React.Fragment, null,
                    child,
                    React.createElement(View, { style: [styles.divider, dividerMargin] })));
            }
            return child;
        })));
};
const getStyles = cacheStyles((theme) => ({
    container: {
        flexDirection: 'column',
        flex: 1
    },
    marginCard: {
        marginVertical: theme.rem(0)
    },
    marginScene: {
        marginVertical: theme.rem(DEFAULT_MARGIN_REM)
    },
    divider: {
        height: theme.thinLineWidth,
        borderBottomWidth: theme.thinLineWidth,
        borderBottomColor: theme.lineDivider
    },
    dividerMarginScene: {
        marginVertical: theme.rem(DEFAULT_MARGIN_REM),
        marginLeft: theme.rem(1),
        marginRight: -theme.rem(DEFAULT_MARGIN_REM)
    },
    dividerMarginCard: {
        margin: theme.rem(DEFAULT_MARGIN_REM)
    }
}));
