import { useMemo } from 'react';
import { useTheme } from '../components/services/ThemeContext';
export const useSpaceStyle = (props) => {
    const theme = useTheme();
    const { around, horizontal, vertical, top, bottom, left, right, expand = false, sideways = false } = props;
    const topFill = boolify(top, vertical, around);
    const bottomFill = boolify(bottom, vertical, around);
    const leftFill = boolify(left, horizontal, around);
    const rightFill = boolify(right, horizontal, around);
    const topUnits = numberify(top, vertical, around);
    const bottomUnits = numberify(bottom, vertical, around);
    const leftUnits = numberify(left, horizontal, around);
    const rightUnits = numberify(right, horizontal, around);
    // Margins:
    const marginTop = theme.rem(topUnits);
    const marginBottom = theme.rem(bottomUnits);
    const marginLeft = theme.rem(leftUnits);
    const marginRight = theme.rem(rightUnits);
    // Direction:
    const flexDirection = sideways ? 'row' : 'column';
    // Alignment:
    const horizontalAlignment = leftFill && rightFill
        ? 'center'
        : rightFill
            ? 'flex-start'
            : leftFill
                ? 'flex-end'
                : undefined;
    const verticalAlignment = topFill && bottomFill
        ? 'center'
        : bottomFill
            ? 'flex-start'
            : topFill
                ? 'flex-end'
                : undefined;
    const alignItems = sideways ? verticalAlignment : horizontalAlignment;
    const justifyContent = sideways
        ? horizontalAlignment !== null && horizontalAlignment !== void 0 ? horizontalAlignment : (expand ? 'space-between' : undefined)
        : verticalAlignment;
    // Flex:
    const flex = expand ? 1 : undefined;
    const style = useMemo(() => ({
        marginTop,
        marginBottom,
        marginLeft,
        marginRight,
        flex,
        flexDirection,
        alignItems,
        justifyContent
    }), [
        alignItems,
        flex,
        flexDirection,
        justifyContent,
        marginBottom,
        marginLeft,
        marginRight,
        marginTop
    ]);
    return style;
};
const numberify = (...things) => {
    for (const thing of things) {
        if (typeof thing === 'number') {
            return thing;
        }
    }
    return 0;
};
const boolify = (...things) => {
    return things.some(thing => {
        return typeof thing === 'boolean' && thing;
    });
};
