var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { asArray, asEither, asJSON, asMaybe, asObject, asOptional, asString, asValue, uncleaner } from 'cleaners';
import { makeReactNativeDisklet } from 'disklet';
/**
 * Looks up a login's status.
 * Returns the location where the secret is stored, false if disabled,
 * or undefined if we have no information about the user.
 */
export function getKeychainStatus(file, account) {
    const accountLoginId = 'rootLoginId' in account ? account.rootLoginId : account.loginId;
    const row = file.find(row => {
        if (row.loginId != null && row.loginId === accountLoginId)
            return true;
        if (row.username != null && row.username === account.username)
            return true;
        return false;
    });
    return row === null || row === void 0 ? void 0 : row.key;
}
/**
 * Changes a login's status on disk.
 * If a login is enabled, pass the key where the secret is stored.
 * If a login is disabled, pass 'false'.
 * To forget a login, pass `undefined`.
 */
export function saveKeychainStatus(file, account, status) {
    return __awaiter(this, void 0, void 0, function* () {
        const newRows = file.filter(row => {
            if (row.loginId != null && row.loginId === account.rootLoginId)
                return false;
            if (row.username != null && row.username === account.username)
                return false;
            return true;
        });
        if (status != null) {
            newRows.unshift({
                loginId: account.rootLoginId,
                key: status
            });
        }
        const text = wasKeychainFile({ logins: newRows });
        yield disklet.setText('fingerprint.json', text);
    });
}
/**
 * Reads all saved login statuses from disk.
 */
export function readKeychainFile() {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const json = yield disklet.getText('fingerprint.json');
            const clean = asMaybeKeychainFile(json);
            if (clean != null)
                return clean.logins;
            const legacy = asLegacyKeychainFile(json);
            const out = [];
            for (const username of legacy.enabledUsers) {
                out.push({
                    key: username + '___key_loginkey',
                    username
                });
            }
            for (const username of legacy.disabledUsers) {
                out.push({
                    key: false,
                    username
                });
            }
            return out;
        }
        catch (error) {
            return [];
        }
    });
}
const disklet = makeReactNativeDisklet();
const asFalse = asValue(false);
const asLegacyKeychainFile = asJSON(asObject({
    enabledUsers: asOptional(asArray(asString), []),
    disabledUsers: asOptional(asArray(asString), [])
}));
const asKeychainFile = asJSON(asObject({
    logins: asArray(asObject({
        key: asEither(asString, asFalse),
        loginId: asOptional(asString),
        username: asOptional(asString)
    }))
}));
const asMaybeKeychainFile = asMaybe(asKeychainFile);
const wasKeychainFile = uncleaner(asKeychainFile);
